@@ -21,10 +21,4 @@ lines = """
 %rehashx
 """
 
-# You have to make sure that attributes that are containers already
-# exist before using them.  Simple assigning a new list will override
-# all previous values.
-if hasattr(app, 'exec_lines'):
-    app.exec_lines.append(lines)
-else:
-    app.exec_lines = [lines]
+app.exec_lines.append(lines)
