@@ -7,12 +7,9 @@ Similar in spirit to the inspect module, but all calls take a name argument to
 reference the name under which an object is being read.
 """
 
-#*****************************************************************************
-#       Copyright (C) 2001-2004 Fernando Perez <fperez@colorado.edu>
-#
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
-#*****************************************************************************
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
+
 from __future__ import print_function
 
 __all__ = ['Inspector','InspectColors']
@@ -21,6 +18,7 @@ __all__ = ['Inspector','InspectColors']
 import inspect
 import linecache
 import os
+from textwrap import dedent
 import types
 import io as stdlib_io
 
@@ -31,22 +29,26 @@ except ImportError:
 
 # IPython's own
 from IPython.core import page
+from IPython.lib.pretty import pretty
 from IPython.testing.skipdoctest import skip_doctest_py3
 from IPython.utils import PyColorize
 from IPython.utils import io
 from IPython.utils import openpy
 from IPython.utils import py3compat
 from IPython.utils.dir2 import safe_hasattr
+from IPython.utils.path import compress_user
 from IPython.utils.text import indent
 from IPython.utils.wildcard import list_namespace
 from IPython.utils.coloransi import TermColors, ColorScheme, ColorSchemeTable
 from IPython.utils.py3compat import cast_unicode, string_types, PY3
+from IPython.utils.signatures import signature
 
 # builtin docstrings to ignore
 _func_call_docstring = types.FunctionType.__call__.__doc__
 _object_init_docstring = object.__init__.__doc__
 _builtin_type_docstrings = {
-    t.__doc__ for t in (types.ModuleType, types.MethodType, types.FunctionType)
+    inspect.getdoc(t) for t in (types.ModuleType, types.MethodType,
+                                types.FunctionType, property)
 }
 
 _builtin_func_type = type(all)
@@ -153,33 +155,68 @@ def getdoc(obj):
         return None
 
 
-def getsource(obj,is_binary=False):
+def getsource(obj, oname=''):
     """Wrapper around inspect.getsource.
 
     This can be modified by other projects to provide customized source
     extraction.
 
-    Inputs:
+    Parameters
+    ----------
+    obj : object
+        an object whose source code we will attempt to extract
+    oname : str
+        (optional) a name under which the object is known
 
-    - obj: an object whose source code we will attempt to extract.
+    Returns
+    -------
+    src : unicode or None
 
-    Optional inputs:
+    """
 
-    - is_binary: whether the object is known to come from a binary source.
-      This implementation will skip returning any output for binary objects, but
-      custom extractors may know how to meaningfully process them."""
+    if isinstance(obj, property):
+        sources = []
+        for attrname in ['fget', 'fset', 'fdel']:
+            fn = getattr(obj, attrname)
+            if fn is not None:
+                encoding = get_encoding(fn)
+                oname_prefix = ('%s.' % oname) if oname else ''
+                sources.append(cast_unicode(
+                    ''.join(('# ', oname_prefix, attrname)),
+                    encoding=encoding))
+                if inspect.isfunction(fn):
+                    sources.append(dedent(getsource(fn)))
+                else:
+                    # Default str/repr only prints function name,
+                    # pretty.pretty prints module name too.
+                    sources.append(cast_unicode(
+                        '%s%s = %s\n' % (
+                            oname_prefix, attrname, pretty(fn)),
+                        encoding=encoding))
+        if sources:
+            return '\n'.join(sources)
+        else:
+            return None
 
-    if is_binary:
-        return None
     else:
-        # get source if obj was decorated with @decorator
-        if hasattr(obj,"__wrapped__"):
+        # Get source for non-property objects.
+
+        # '__wrapped__' attribute is used by some decorators (e.g. ones defined
+        # functools) to provide access to the decorated function.
+        if hasattr(obj, "__wrapped__"):
             obj = obj.__wrapped__
+
         try:
             src = inspect.getsource(obj)
         except TypeError:
-            if hasattr(obj,'__class__'):
-                src = inspect.getsource(obj.__class__)
+            # The object itself provided no meaningful source, try looking for
+            # its class definition instead.
+            if hasattr(obj, '__class__'):
+                try:
+                    src = inspect.getsource(obj.__class__)
+                except TypeError:
+                    return None
+
         encoding = get_encoding(obj)
         return cast_unicode(src, encoding=encoding)
 
@@ -355,7 +392,7 @@ class Inspector:
         If any exception is generated, None is returned instead and the
         exception is suppressed."""
         try:
-            hdef = oname + inspect.formatargspec(*getargspec(obj))
+            hdef = oname + str(signature(obj))
             return cast_unicode(hdef)
         except:
             return None
@@ -460,15 +497,18 @@ class Inspector:
         else:
             page.page('\n'.join(lines))
 
-    def psource(self,obj,oname=''):
+    def psource(self, obj, oname=''):
         """Print the source code for an object."""
 
         # Flush the source cache because inspect can return out-of-date source
         linecache.checkcache()
         try:
-            src = getsource(obj)
-        except:
-            self.noinfo('source',oname)
+            src = getsource(obj, oname=oname)
+        except Exception:
+            src = None
+
+        if src is None:
+            self.noinfo('source', oname)
         else:
             page.page(self.format(src))
 
@@ -515,25 +555,84 @@ class Inspector:
                 title = header((title+":").ljust(title_width))
             out.append(cast_unicode(title) + cast_unicode(content))
         return "\n".join(out)
+    
+    def _format_info(self, obj, oname='', formatter=None, info=None, detail_level=0):
+        """Format an info dict as text"""
+        info = self.info(obj, oname=oname, formatter=formatter,
+                            info=info, detail_level=detail_level)
+        displayfields = []
+        def add_fields(fields):
+            for title, key in fields:
+                field = info[key]
+                if field is not None:
+                    displayfields.append((title, field.rstrip()))
 
-    # The fields to be displayed by pinfo: (fancy_name, key_in_info_dict)
-    pinfo_fields1 = [("Type", "type_name"),
-                    ]
-                    
-    pinfo_fields2 = [("String form", "string_form"),
-                    ]
+        if info['isalias']:
+            add_fields([('Repr', "string_form")])
 
-    pinfo_fields3 = [("Length", "length"),
-                    ("File", "file"),
-                    ("Definition", "definition"),
-                    ]
+        elif info['ismagic']:
+            if detail_level > 0 and info['source'] is not None:
+                add_fields([("Source", "source")])
+            else:
+                add_fields([("Docstring", "docstring")])
 
-    pinfo_fields_obj = [("Class docstring", "class_docstring"),
+            add_fields([("File", "file"),
+                       ])
+
+        elif info['isclass'] or is_simple_callable(obj):
+            # Functions, methods, classes
+            add_fields([("Signature", "definition"),
+                        ("Init signature", "init_definition"),
+                       ])
+            if detail_level > 0 and info['source'] is not None:
+                add_fields([("Source", "source")])
+            else:
+                add_fields([("Docstring", "docstring"),
+                            ("Init docstring", "init_docstring"),
+                           ])
+
+            add_fields([('File', 'file'),
+                        ('Type', 'type_name'),
+                       ])
+
+        else:
+            # General Python objects
+            add_fields([("Type", "type_name")])
+
+            # Base class for old-style instances
+            if (not py3compat.PY3) and isinstance(obj, types.InstanceType) and info['base_class']:
+                displayfields.append(("Base Class", info['base_class'].rstrip()))
+
+            add_fields([("String form", "string_form")])
+
+            # Namespace
+            if info['namespace'] != 'Interactive':
+                displayfields.append(("Namespace", info['namespace'].rstrip()))
+
+            add_fields([("Length", "length"),
+                        ("File", "file"),
+                        ("Signature", "definition"),
+                       ])
+
+            # Source or docstring, depending on detail level and whether
+            # source found.
+            if detail_level > 0 and info['source'] is not None:
+                displayfields.append(("Source",
+                                      self.format(cast_unicode(info['source']))))
+            elif info['docstring'] is not None:
+                displayfields.append(("Docstring", info["docstring"]))
+
+            add_fields([("Class docstring", "class_docstring"),
                         ("Init docstring", "init_docstring"),
-                        ("Call def", "call_def"),
-                        ("Call docstring", "call_docstring")]
-
-    def pinfo(self,obj,oname='',formatter=None,info=None,detail_level=0):
+                        ("Call signature", "call_def"),
+                        ("Call docstring", "call_docstring")])
+        
+        if displayfields:
+            return self._format_fields(displayfields)
+        else:
+            return u''
+    
+    def pinfo(self, obj, oname='', formatter=None, info=None, detail_level=0):
         """Show detailed information about an object.
 
         Optional arguments:
@@ -547,54 +646,10 @@ class Inspector:
 
         - detail_level: if set to 1, more information is given.
         """
-        info = self.info(obj, oname=oname, formatter=formatter,
-                            info=info, detail_level=detail_level)
-        displayfields = []
-        def add_fields(fields):
-            for title, key in fields:
-                field = info[key]
-                if field is not None:
-                    displayfields.append((title, field.rstrip()))
-        
-        add_fields(self.pinfo_fields1)
-        
-        # Base class for old-style instances
-        if (not py3compat.PY3) and isinstance(obj, types.InstanceType) and info['base_class']:
-            displayfields.append(("Base Class", info['base_class'].rstrip()))
-        
-        add_fields(self.pinfo_fields2)
-        
-        # Namespace
-        if info['namespace'] != 'Interactive':
-            displayfields.append(("Namespace", info['namespace'].rstrip()))
-
-        add_fields(self.pinfo_fields3)
-        if info['isclass'] and info['init_definition']:
-            displayfields.append(("Init definition",
-                            info['init_definition'].rstrip()))
-        
-        # Source or docstring, depending on detail level and whether
-        # source found.
-        if detail_level > 0 and info['source'] is not None:
-            displayfields.append(("Source", 
-                                  self.format(cast_unicode(info['source']))))
-        elif info['docstring'] is not None:
-            displayfields.append(("Docstring", info["docstring"]))
-
-        # Constructor info for classes
-        if info['isclass']:
-            if info['init_docstring'] is not None:
-                displayfields.append(("Init docstring",
-                                    info['init_docstring']))
-
-        # Info for objects:
-        else:
-            add_fields(self.pinfo_fields_obj)
-
-        # Finally send to printer/pager:
-        if displayfields:
-            page.page(self._format_fields(displayfields))
-
+        text = self._format_info(obj, oname, formatter, info, detail_level)
+        if text:
+            page.page(text)
+    
     def info(self, obj, oname='', formatter=None, info=None, detail_level=0):
         """Compute a dict with detailed information about an object.
 
@@ -691,33 +746,26 @@ class Inspector:
                 binary_file = True
             elif fname.endswith('<string>'):
                 fname = 'Dynamically generated function. No source code available.'
-            out['file'] = fname
-        
-        # Docstrings only in detail 0 mode, since source contains them (we
-        # avoid repetitions).  If source fails, we add them back, see below.
-        if ds and detail_level == 0:
-                out['docstring'] = ds
+            out['file'] = compress_user(fname)
 
-        # Original source code for any callable
+        # Original source code for a callable, class or property.
         if detail_level:
             # Flush the source cache because inspect can return out-of-date
             # source
             linecache.checkcache()
-            source = None
             try:
-                try:
-                    source = getsource(obj, binary_file)
-                except TypeError:
-                    if hasattr(obj, '__class__'):
-                        source = getsource(obj.__class__, binary_file)
-                if source is not None:
-                    out['source'] = source.rstrip()
+                if isinstance(obj, property) or not binary_file:
+                    src = getsource(obj, oname)
+                    if src is not None:
+                        src = src.rstrip()
+                    out['source'] = src
+
             except Exception:
                 pass
 
-            if ds and source is None:
-                out['docstring'] = ds
-
+        # Add docstring only if no source is to be shown (avoid repetitions).
+        if ds and out.get('source', None) is None:
+            out['docstring'] = ds
 
         # Constructor docstring for classes
         if inspect.isclass(obj):
@@ -820,7 +868,6 @@ class Inspector:
 
         return object_info(**out)
 
-
     def psearch(self,pattern,ns_table,ns_search=[],
                 ignore_case=False,show_all=False):
         """Search namespaces with wildcards for objects.
