@@ -1,25 +1,13 @@
 """Tests for HTMLExporter"""
 
-#-----------------------------------------------------------------------------
-# Copyright (c) 2013, the IPython Development Team.
-#
+# Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
-#
-# The full license is in the file COPYING.txt, distributed with this software.
-#-----------------------------------------------------------------------------
-
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
 
 from .base import ExportersTestsBase
 from ..html import HTMLExporter
-from IPython.testing.decorators import onlyif_any_cmd_exists
+from IPython.nbformat import v4
 import re
 
-#-----------------------------------------------------------------------------
-# Class
-#-----------------------------------------------------------------------------
 
 class TestHTMLExporter(ExportersTestsBase):
     """Tests for HTMLExporter"""
@@ -34,7 +22,6 @@ class TestHTMLExporter(ExportersTestsBase):
         HTMLExporter()
 
 
-    @onlyif_any_cmd_exists('nodejs', 'node', 'pandoc')
     def test_export(self):
         """
         Can a HTMLExporter export something?
@@ -43,7 +30,6 @@ class TestHTMLExporter(ExportersTestsBase):
         assert len(output) > 0
 
 
-    @onlyif_any_cmd_exists('nodejs', 'node', 'pandoc')
     def test_export_basic(self):
         """
         Can a HTMLExporter export using the 'basic' template?
@@ -52,7 +38,6 @@ class TestHTMLExporter(ExportersTestsBase):
         assert len(output) > 0
 
 
-    @onlyif_any_cmd_exists('nodejs', 'node', 'pandoc')
     def test_export_full(self):
         """
         Can a HTMLExporter export using the 'full' template?
@@ -60,7 +45,6 @@ class TestHTMLExporter(ExportersTestsBase):
         (output, resources) = HTMLExporter(template_file='full').from_filename(self._get_notebook())
         assert len(output) > 0
 
-    @onlyif_any_cmd_exists('nodejs', 'node', 'pandoc')
     def test_prompt_number(self):
         """
         Does HTMLExporter properly format input and output prompts?
@@ -76,7 +60,6 @@ class TestHTMLExporter(ExportersTestsBase):
         assert re.findall(in_regex, output) == ins
         assert re.findall(out_regex, output) == outs
 
-    @onlyif_any_cmd_exists('nodejs', 'node', 'pandoc')
     def test_png_metadata(self):
         """
         Does HTMLExporter with the 'basic' template treat pngs with width/height metadata correctly?
@@ -84,3 +67,19 @@ class TestHTMLExporter(ExportersTestsBase):
         (output, resources) = HTMLExporter(template_file='basic').from_filename(
             self._get_notebook(nb_name="pngmetadata.ipynb"))
         assert len(output) > 0
+
+    def test_javascript_output(self):
+        nb = v4.new_notebook(
+            cells=[
+                v4.new_code_cell(
+                    outputs=[v4.new_output(
+                        output_type='display_data',
+                        data={
+                            'application/javascript': "javascript_output();"
+                        }
+                    )]
+                )
+            ]
+        )
+        (output, resources) = HTMLExporter(template_file='basic').from_notebook_node(nb)
+        self.assertIn('javascript_output', output)
