@@ -1,21 +1,12 @@
 #!/usr/bin/env python
-"""NBConvert is a utility for conversion of .ipynb files.
+"""NbConvert is a utility for conversion of .ipynb files.
 
 Command-line interface for the NbConvert conversion utility.
 """
-#-----------------------------------------------------------------------------
-#Copyright (c) 2013, the IPython Development Team.
-#
-#Distributed under the terms of the Modified BSD License.
-#
-#The full license is in the file COPYING.txt, distributed with this software.
-#-----------------------------------------------------------------------------
 
-#-----------------------------------------------------------------------------
-#Imports
-#-----------------------------------------------------------------------------
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
 
-# Stdlib imports
 from __future__ import print_function
 
 import logging
@@ -23,15 +14,13 @@ import sys
 import os
 import glob
 
-# From IPython
 from IPython.core.application import BaseIPythonApplication, base_aliases, base_flags
 from IPython.core.profiledir import ProfileDir
 from IPython.config import catch_config_error, Configurable
 from IPython.utils.traitlets import (
-    Unicode, List, Instance, DottedObjectName, Type, CaselessStrEnum,
+    Unicode, List, Instance, DottedObjectName, Type, CaselessStrEnum, Bool,
 )
 from IPython.utils.importstring import import_item
-from IPython.utils.text import dedent
 
 from .exporters.export import get_export_names, exporter_map
 from IPython.nbconvert import exporters, preprocessors, writers, postprocessors
@@ -64,14 +53,27 @@ nbconvert_aliases.update({
     'post': 'NbConvertApp.postprocessor_class',
     'output': 'NbConvertApp.output_base',
     'reveal-prefix': 'RevealHelpPreprocessor.url_prefix',
+    'nbformat': 'NotebookExporter.nbformat_version',
 })
 
 nbconvert_flags = {}
 nbconvert_flags.update(base_flags)
 nbconvert_flags.update({
+    'execute' : (
+        {'ExecutePreprocessor' : {'enabled' : True}},
+        "Execute the notebook prior to export."
+        ),
     'stdout' : (
         {'NbConvertApp' : {'writer_class' : "StdoutWriter"}},
         "Write notebook output to stdout instead of files."
+        ),
+    'inplace' : (
+        {
+            'NbConvertApp' : {'use_output_suffix' : False},
+            'FilesWriter': {'build_directory': ''}
+        },
+        """Run nbconvert in place, overwriting the existing notebook (only 
+        relevant when converting to notebook format)"""
         )
 })
 
@@ -103,9 +105,16 @@ class NbConvertApp(BaseIPythonApplication):
         WARNING: THE COMMANDLINE INTERFACE MAY CHANGE IN FUTURE RELEASES.""")
 
     output_base = Unicode('', config=True, help='''overwrite base name use for output files.
-            can only  be use when converting one notebook at a time.
+            can only be used when converting one notebook at a time.
             ''')
 
+    use_output_suffix = Bool(
+        True, 
+        config=True,
+        help="""Whether to apply a suffix prior to the extension (only relevant
+            when converting to notebook format). The suffix is determined by
+            the exporter, and is usually '.nbconvert'.""")
+
     examples = Unicode(u"""
         The simplest way to use nbconvert is
         
@@ -128,9 +137,9 @@ class NbConvertApp(BaseIPythonApplication):
         
         > ipython nbconvert mynotebook.ipynb --stdout
 
-        A post-processor can be used to compile a PDF
+        PDF is generated via latex
 
-        > ipython nbconvert mynotebook.ipynb --to latex --post PDF
+        > ipython nbconvert mynotebook.ipynb --to pdf
         
         You can get (and serve) a Reveal.js-powered slideshow
         
@@ -174,8 +183,7 @@ class NbConvertApp(BaseIPythonApplication):
     postprocessor_class = DottedOrNone(config=True, 
                                     help="""PostProcessor class used to write the 
                                     results of the conversion""")
-    postprocessor_aliases = {'pdf': 'IPython.nbconvert.postprocessors.pdf.PDFPostProcessor',
-                              'serve': 'IPython.nbconvert.postprocessors.serve.ServePostProcessor'}
+    postprocessor_aliases = {'serve': 'IPython.nbconvert.postprocessors.serve.ServePostProcessor'}
     postprocessor_factory = Type()
 
     def _postprocessor_class_changed(self, name, old, new):
@@ -250,6 +258,8 @@ class NbConvertApp(BaseIPythonApplication):
         """
         self._writer_class_changed(None, self.writer_class, self.writer_class)
         self.writer = self.writer_factory(parent=self)
+        if hasattr(self.writer, 'build_directory') and self.writer.build_directory != '':
+            self.use_output_suffix = False
 
     def init_postprocessor(self):
         """
@@ -267,54 +277,128 @@ class NbConvertApp(BaseIPythonApplication):
         super(NbConvertApp, self).start()
         self.convert_notebooks()
 
+    def init_single_notebook_resources(self, notebook_filename):
+        """Step 1: Initialize resources
+
+        This intializes the resources dictionary for a single notebook. This
+        method should return the resources dictionary, and MUST include the
+        following keys:
+
+            - profile_dir: the location of the profile directory
+            - unique_key: the notebook name
+            - output_files_dir: a directory where output files (not including
+              the notebook itself) should be saved
+
+        """
+
+        # Get a unique key for the notebook and set it in the resources object.
+        basename = os.path.basename(notebook_filename)
+        notebook_name = basename[:basename.rfind('.')]
+        if self.output_base:
+            # strip duplicate extension from output_base, to avoid Basname.ext.ext
+            if getattr(self.exporter, 'file_extension', False):
+                base, ext = os.path.splitext(self.output_base)
+                if ext == self.exporter.file_extension:
+                    self.output_base = base
+            notebook_name = self.output_base
+
+        self.log.debug("Notebook name is '%s'", notebook_name)
+
+        # first initialize the resources we want to use
+        resources = {}
+        resources['profile_dir'] = self.profile_dir.location
+        resources['unique_key'] = notebook_name
+        resources['output_files_dir'] = '%s_files' % notebook_name
+
+        return resources
+
+    def export_single_notebook(self, notebook_filename, resources):
+        """Step 2: Export the notebook
+
+        Exports the notebook to a particular format according to the specified
+        exporter. This function returns the output and (possibly modified)
+        resources from the exporter.
+
+        """
+        try:
+            output, resources = self.exporter.from_filename(notebook_filename, resources=resources)
+        except ConversionException:
+            self.log.error("Error while converting '%s'", notebook_filename, exc_info=True)
+            self.exit(1)
+
+        return output, resources
+
+    def write_single_notebook(self, output, resources):
+        """Step 3: Write the notebook to file
+
+        This writes output from the exporter to file using the specified writer.
+        It returns the results from the writer.
+
+        """
+        if 'unique_key' not in resources:
+            raise KeyError("unique_key MUST be specified in the resources, but it is not")
+
+        notebook_name = resources['unique_key']
+        if self.use_output_suffix and not self.output_base:
+            notebook_name += resources.get('output_suffix', '')
+
+        write_results = self.writer.write(
+            output, resources, notebook_name=notebook_name)
+        return write_results
+
+    def postprocess_single_notebook(self, write_results):
+        """Step 4: Postprocess the notebook
+
+        This postprocesses the notebook after it has been written, taking as an
+        argument the results of writing the notebook to file. This only actually
+        does anything if a postprocessor has actually been specified.
+
+        """
+        # Post-process if post processor has been defined.
+        if hasattr(self, 'postprocessor') and self.postprocessor:
+            self.postprocessor(write_results)
+
+    def convert_single_notebook(self, notebook_filename):
+        """Convert a single notebook. Performs the following steps:
+
+            1. Initialize notebook resources
+            2. Export the notebook to a particular format
+            3. Write the exported notebook to file
+            4. (Maybe) postprocess the written file
+
+        """
+        self.log.info("Converting notebook %s to %s", notebook_filename, self.export_format)
+        resources = self.init_single_notebook_resources(notebook_filename)
+        output, resources = self.export_single_notebook(notebook_filename, resources)
+        write_results = self.write_single_notebook(output, resources)
+        self.postprocess_single_notebook(write_results)
+
     def convert_notebooks(self):
         """
         Convert the notebooks in the self.notebook traitlet
         """
-        # Export each notebook
-        conversion_success = 0
-
+        # check that the output base isn't specified if there is more than
+        # one notebook to convert
         if self.output_base != '' and len(self.notebooks) > 1:
             self.log.error(
-            """UsageError: --output flag or `NbConvertApp.output_base` config option
-            cannot be used when converting multiple notebooks.
-            """)
+                """
+                UsageError: --output flag or `NbConvertApp.output_base` config option
+                cannot be used when converting multiple notebooks.
+                """
+            )
             self.exit(1)
         
-        exporter = exporter_map[self.export_format](config=self.config)
+        # initialize the exporter
+        self.exporter = exporter_map[self.export_format](config=self.config)
 
-        for notebook_filename in self.notebooks:
-            self.log.info("Converting notebook %s to %s", notebook_filename, self.export_format)
-
-            # Get a unique key for the notebook and set it in the resources object.
-            basename = os.path.basename(notebook_filename)
-            notebook_name = basename[:basename.rfind('.')]
-            if self.output_base:
-                notebook_name = self.output_base
-            resources = {}
-            resources['unique_key'] = notebook_name
-            resources['output_files_dir'] = '%s_files' % notebook_name
-            self.log.info("Support files will be in %s", os.path.join(resources['output_files_dir'], ''))
-
-            # Try to export
-            try:
-                output, resources = exporter.from_filename(notebook_filename, resources=resources)
-            except ConversionException as e:
-                self.log.error("Error while converting '%s'", notebook_filename,
-                      exc_info=True)
-                self.exit(1)
-            else:
-                write_resultes = self.writer.write(output, resources, notebook_name=notebook_name)
-
-                #Post-process if post processor has been defined.
-                if hasattr(self, 'postprocessor') and self.postprocessor:
-                    self.postprocessor(write_resultes)
-                conversion_success += 1
-
-        # If nothing was converted successfully, help the user.
-        if conversion_success == 0:
+        # no notebooks to convert!
+        if len(self.notebooks) == 0:
             self.print_help()
             sys.exit(-1)
+
+        # convert each notebook
+        for notebook_filename in self.notebooks:
+            self.convert_single_notebook(notebook_filename)
             
 #-----------------------------------------------------------------------------
 # Main entry point
