@@ -1,24 +1,15 @@
-"""
-Contains writer for writing nbconvert output to filesystem.
-"""
-#-----------------------------------------------------------------------------
-#Copyright (c) 2013, the IPython Development Team.
-#
-#Distributed under the terms of the Modified BSD License.
-#
-#The full license is in the file COPYING.txt, distributed with this software.
-#-----------------------------------------------------------------------------
+"""Contains writer for writing nbconvert output to filesystem."""
 
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
 
 import io
 import os
 import glob
 
 from IPython.utils.traitlets import Unicode
-from IPython.utils.path import link_or_copy
+from IPython.utils.path import link_or_copy, ensure_dir_exists
+from IPython.utils.py3compat import unicode_type
 
 from .base import WriterBase
 
@@ -34,11 +25,19 @@ class FilesWriter(WriterBase):
                               help="""Directory to write output to.  Leave blank
                               to output to the current directory""")
 
+    relpath = Unicode(
+        "", config=True, 
+        help="""When copying files that the notebook depends on, copy them in
+        relation to this path, such that the destination filename will be
+        os.path.relpath(filename, relpath). If FilesWriter is operating on a
+        notebook that already exists elsewhere on disk, then the default will be
+        the directory containing that notebook.""")
+
 
     # Make sure that the output directory exists.
     def _build_directory_changed(self, name, old, new):
-        if new and not os.path.isdir(new):
-            os.makedirs(new)
+        if new:
+            ensure_dir_exists(new)
 
 
     def __init__(self, **kw):
@@ -48,9 +47,9 @@ class FilesWriter(WriterBase):
     
     def _makedir(self, path):
         """Make a directory if it doesn't already exist"""
-        if path and not os.path.isdir(path):
+        if path:
             self.log.info("Making directory %s", path)
-            os.makedirs(path)
+            ensure_dir_exists(path)
 
     def write(self, output, resources, notebook_name=None, **kw):
             """
@@ -68,10 +67,20 @@ class FilesWriter(WriterBase):
             # Pull the extension and subdir from the resources dict.
             output_extension = resources.get('output_extension', None)
 
+            # Get the relative path for copying files
+            if self.relpath == '':
+                relpath = resources.get('metadata', {}).get('path', '')
+            else:
+                relpath = self.relpath
+
             # Write all of the extracted resources to the destination directory.
             # NOTE: WE WRITE EVERYTHING AS-IF IT'S BINARY.  THE EXTRACT FIG
             # PREPROCESSOR SHOULD HANDLE UNIX/WINDOWS LINE ENDINGS...
-            for filename, data in resources.get('outputs', {}).items():
+
+            items = resources.get('outputs', {}).items()
+            if items:
+                self.log.info("Support files will be in %s", os.path.join(resources.get('output_files_dir',''), ''))
+            for filename, data in items:
 
                 # Determine where to write the file to
                 dest = os.path.join(self.build_directory, filename)
@@ -90,8 +99,14 @@ class FilesWriter(WriterBase):
                     # Copy files that match search pattern
                     for matching_filename in glob.glob(filename):
 
+                        # compute the relative path for the filename
+                        if relpath != '':
+                            dest_filename = os.path.relpath(matching_filename, relpath)
+                        else:
+                            dest_filename = matching_filename
+
                         # Make sure folder exists.
-                        dest = os.path.join(self.build_directory, filename)
+                        dest = os.path.join(self.build_directory, dest_filename)
                         path = os.path.dirname(dest)
                         self._makedir(path)
 
@@ -102,7 +117,7 @@ class FilesWriter(WriterBase):
 
             # Determine where to write conversion results.
             if output_extension is not None:
-                dest = notebook_name + '.' + output_extension
+                dest = notebook_name + output_extension
             else:
                 dest = notebook_name
             if self.build_directory:
@@ -110,6 +125,11 @@ class FilesWriter(WriterBase):
 
             # Write conversion results.
             self.log.info("Writing %i bytes to %s", len(output), dest)
-            with io.open(dest, 'w', encoding='utf-8') as f:
-                f.write(output)
+            if isinstance(output, unicode_type):
+                with io.open(dest, 'w', encoding='utf-8') as f:
+                    f.write(output)
+            else:
+                with io.open(dest, 'wb') as f:
+                    f.write(output)
+                
             return dest
