@@ -3,11 +3,9 @@
 This supports the additional functionality provided by the IPython kernel.
 """
 
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
 
-# Standard library imports
 from collections import namedtuple
 import os.path
 import re
@@ -16,11 +14,10 @@ import sys
 import time
 from textwrap import dedent
 
-# System library imports
 from IPython.external.qt import QtCore, QtGui
 
-# Local imports
 from IPython.core.inputsplitter import IPythonInputSplitter
+from IPython.core.release import version
 from IPython.core.inputtransformer import ipy_prompt
 from IPython.utils.traitlets import Bool, Unicode
 from .frontend_widget import FrontendWidget
@@ -111,6 +108,7 @@ class IPythonWidget(FrontendWidget):
     _payload_source_next_input = 'set_next_input'
     _payload_source_page = 'page'
     _retrying_history_request = False
+    _starting = False
 
     #---------------------------------------------------------------------------
     # 'object' interface
@@ -148,23 +146,28 @@ class IPythonWidget(FrontendWidget):
         info = self._request_info.get('complete')
         if info and info.id == rep['parent_header']['msg_id'] and \
                 info.pos == cursor.position():
-            matches = rep['content']['matches']
-            text = rep['content']['matched_text']
-            offset = len(text)
+            content = rep['content']
+            matches = content['matches']
+            start = content['cursor_start']
+            end = content['cursor_end']
+            
+            start = max(start, 0)
+            end = max(end, start)
 
-            # Clean up matches with period and path separators if the matched
-            # text has not been transformed. This is done by truncating all
-            # but the last component and then suitably decreasing the offset
-            # between the current cursor position and the start of completion.
-            if len(matches) > 1 and matches[0][:offset] == text:
-                parts = re.split(r'[./\\]', text)
-                sep_count = len(parts) - 1
-                if sep_count:
-                    chop_length = sum(map(len, parts[:sep_count])) + sep_count
-                    matches = [ match[chop_length:] for match in matches ]
-                    offset -= chop_length
+            # Move the control's cursor to the desired end point
+            cursor_pos = self._get_input_buffer_cursor_pos()
+            if end < cursor_pos:
+                cursor.movePosition(QtGui.QTextCursor.Left,
+                                    n=(cursor_pos - end))
+            elif end > cursor_pos:
+                cursor.movePosition(QtGui.QTextCursor.Right,
+                                    n=(end - cursor_pos))
+            # This line actually applies the move to control's cursor
+            self._control.setTextCursor(cursor)
 
-            # Move the cursor to the start of the match and complete.
+            offset = end - start
+            # Move the local cursor object to the start of the match and
+            # complete.
             cursor.movePosition(QtGui.QTextCursor.Left, n=offset)
             self._complete_with_items(cursor, matches)
 
@@ -200,7 +203,7 @@ class IPythonWidget(FrontendWidget):
                 self._retrying_history_request = True
                 # wait out the kernel's queue flush, which is currently timed at 0.1s
                 time.sleep(0.25)
-                self.kernel_client.shell_channel.history(hist_access_type='tail',n=1000)
+                self.kernel_client.history(hist_access_type='tail',n=1000)
             else:
                 self._retrying_history_request = False
             return
@@ -216,23 +219,33 @@ class IPythonWidget(FrontendWidget):
                 items.append(cell)
                 last_cell = cell
         self._set_history(items)
+    
+    def _insert_other_input(self, cursor, content):
+        """Insert function for input from other frontends"""
+        cursor.beginEditBlock()
+        start = cursor.position()
+        n = content.get('execution_count', 0)
+        cursor.insertText('\n')
+        self._insert_html(cursor, self._make_in_prompt(n))
+        cursor.insertText(content['code'])
+        self._highlighter.rehighlightBlock(cursor.block())
+        cursor.endEditBlock()
+        
+    def _handle_execute_input(self, msg):
+        """Handle an execute_input message"""
+        self.log.debug("execute_input: %s", msg.get('content', ''))
+        if self.include_output(msg):
+            self._append_custom(self._insert_other_input, msg['content'], before_prompt=True)
 
-    def _handle_pyout(self, msg):
-        """ Reimplemented for IPython-style "display hook".
-        """
-        self.log.debug("pyout: %s", msg.get('content', ''))
-        if not self._hidden and self._is_from_this_session(msg):
+    
+    def _handle_execute_result(self, msg):
+        """Reimplemented for IPython-style "display hook"."""
+        if self.include_output(msg):
             self.flush_clearoutput()
             content = msg['content']
             prompt_number = content.get('execution_count', 0)
             data = content['data']
-            if 'text/html' in data:
-                self._append_plain_text(self.output_sep, True)
-                self._append_html(self._make_out_prompt(prompt_number), True)
-                html = data['text/html']
-                self._append_plain_text('\n', True)
-                self._append_html(html + self.output_sep2, True)
-            elif 'text/plain' in data:
+            if 'text/plain' in data:
                 self._append_plain_text(self.output_sep, True)
                 self._append_html(self._make_out_prompt(prompt_number), True)
                 text = data['text/plain']
@@ -243,57 +256,47 @@ class IPythonWidget(FrontendWidget):
                 self._append_plain_text(text + self.output_sep2, True)
 
     def _handle_display_data(self, msg):
-        """ The base handler for the ``display_data`` message.
-        """
-        self.log.debug("display: %s", msg.get('content', ''))
+        """The base handler for the ``display_data`` message."""
         # For now, we don't display data from other frontends, but we
         # eventually will as this allows all frontends to monitor the display
         # data. But we need to figure out how to handle this in the GUI.
-        if not self._hidden and self._is_from_this_session(msg):
+        if self.include_output(msg):
             self.flush_clearoutput()
-            source = msg['content']['source']
             data = msg['content']['data']
             metadata = msg['content']['metadata']
             # In the regular IPythonWidget, we simply print the plain text
             # representation.
-            if 'text/html' in data:
-                html = data['text/html']
-                self._append_html(html, True)
-            elif 'text/plain' in data:
+            if 'text/plain' in data:
                 text = data['text/plain']
                 self._append_plain_text(text, True)
             # This newline seems to be needed for text and html output.
             self._append_plain_text(u'\n', True)
 
     def _handle_kernel_info_reply(self, rep):
-        """ Handle kernel info replies.
-        """
+        """Handle kernel info replies."""
+        content = rep['content']
         if not self._guiref_loaded:
-            if rep['content'].get('language') == 'python':
+            if content.get('implementation') == 'ipython':
                 self._load_guiref_magic()
             self._guiref_loaded = True
+        
+        self.kernel_banner = content.get('banner', '')
+        if self._starting:
+            # finish handling started channels
+            self._starting = False
+            super(IPythonWidget, self)._started_channels()
 
     def _started_channels(self):
         """Reimplemented to make a history request and load %guiref."""
-        super(IPythonWidget, self)._started_channels()
-
+        self._starting = True
         # The reply will trigger %guiref load provided language=='python'
         self.kernel_client.kernel_info()
 
-        self.kernel_client.shell_channel.history(hist_access_type='tail',
-                                                  n=1000)
-    
-    def _started_kernel(self):
-        """Load %guiref when the kernel starts (if channels are also started).
-        
-        Principally triggered by kernel restart.
-        """
-        if self.kernel_client.shell_channel is not None:
-            self._load_guiref_magic()
+        self.kernel_client.history(hist_access_type='tail', n=1000)
     
     def _load_guiref_magic(self):
         """Load %guiref magic."""
-        self.kernel_client.shell_channel.execute('\n'.join([
+        self.kernel_client.execute('\n'.join([
             "try:",
             "    _usage",
             "except:",
@@ -340,24 +343,6 @@ class IPythonWidget(FrontendWidget):
     # 'FrontendWidget' protected interface
     #---------------------------------------------------------------------------
 
-    def _complete(self):
-        """ Reimplemented to support IPython's improved completion machinery.
-        """
-        # We let the kernel split the input line, so we *always* send an empty
-        # text field. Readline-based frontends do get a real text field which
-        # they can use.
-        text = ''
-
-        # Send the completion request to the kernel
-        msg_id = self.kernel_client.shell_channel.complete(
-            text,                                    # text
-            self._get_input_buffer_cursor_line(),    # line
-            self._get_input_buffer_cursor_column(),  # cursor_pos
-            self.input_buffer)                       # block
-        pos = self._get_cursor().position()
-        info = self._CompletionRequest(msg_id, pos)
-        self._request_info['complete'] = info
-
     def _process_execute_error(self, msg):
         """ Reimplemented for IPython-style traceback formatting.
         """
@@ -395,7 +380,7 @@ class IPythonWidget(FrontendWidget):
         """
         # If a number was not specified, make a prompt number request.
         if number is None:
-            msg_id = self.kernel_client.shell_channel.execute('', silent=True)
+            msg_id = self.kernel_client.execute('', silent=True)
             info = self._ExecutionRequest(msg_id, 'prompt')
             self._request_info['execute'][msg_id] = info
             return
@@ -526,7 +511,8 @@ class IPythonWidget(FrontendWidget):
             body = self.in_prompt % number
         except TypeError:
             # allow in_prompt to leave out number, e.g. '>>> '
-            body = self.in_prompt
+            from xml.sax.saxutils import escape
+            body = escape(self.in_prompt)
         return '<span class="in-prompt">%s</span>' % body
 
     def _make_continuation_prompt(self, prompt):
@@ -541,7 +527,12 @@ class IPythonWidget(FrontendWidget):
     def _make_out_prompt(self, number):
         """ Given a prompt number, returns an HTML Out prompt.
         """
-        body = self.out_prompt % number
+        try:
+            body = self.out_prompt % number
+        except TypeError:
+            # allow out_prompt to leave out number, e.g. '<<< '
+            from xml.sax.saxutils import escape
+            body = escape(self.out_prompt)
         return '<span class="out-prompt">%s</span>' % body
 
     #------ Payload handlers --------------------------------------------------
@@ -564,10 +555,11 @@ class IPythonWidget(FrontendWidget):
         # Since the plain text widget supports only a very small subset of HTML
         # and we have no control over the HTML source, we only page HTML
         # payloads in the rich text widget.
-        if item['html'] and self.kind == 'rich':
-            self._page(item['html'], html=True)
+        data = item['data']
+        if 'text/html' in data and self.kind == 'rich':
+            self._page(data['text/html'], html=True)
         else:
-            self._page(item['text'], html=False)
+            self._page(data['text/plain'], html=False)
 
     #------ Trait change handlers --------------------------------------------
 
@@ -599,5 +591,4 @@ class IPythonWidget(FrontendWidget):
     #------ Trait default initializers -----------------------------------------
 
     def _banner_default(self):
-        from IPython.core.usage import default_gui_banner
-        return default_gui_banner
+        return "IPython QtConsole {version}\n".format(version=version)
