@@ -1,6 +1,9 @@
-"""Adapt readline completer interface to make ZMQ request.
-"""
 # -*- coding: utf-8 -*-
+"""Adapt readline completer interface to make ZMQ request."""
+
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
+
 try:
     from queue import Empty  # Py 3
 except ImportError:
@@ -8,6 +11,7 @@ except ImportError:
 
 from IPython.config import Configurable
 from IPython.core.completer import IPCompleter
+from IPython.utils.py3compat import str_to_unicode, unicode_to_str, cast_bytes, cast_unicode
 from IPython.utils.traitlets import Float
 import IPython.utils.rlineimpl as readline
 
@@ -26,19 +30,38 @@ class ZMQCompleter(IPCompleter):
         self.shell = shell
         self.client =  client
         self.matches = []
+        # don't do any splitting client-side,
+        # rely on the kernel for that
+        self.splitter.delims = '\r\n'
+        if self.readline:
+            self.readline.set_completer_delims('\r\n')
+    
+    def complete_request(self, text):
+        line = str_to_unicode(readline.get_line_buffer())
+        byte_cursor_pos = readline.get_endidx()
         
-    def complete_request(self,text):
-        line = readline.get_line_buffer()
-        cursor_pos = readline.get_endidx()
+        # get_endidx is a byte offset
+        # account for multi-byte characters to get correct cursor_pos
+        bytes_before_cursor = cast_bytes(line)[:byte_cursor_pos]
+        cursor_pos = len(cast_unicode(bytes_before_cursor))
         
         # send completion request to kernel
-        # Give the kernel up to 0.5s to respond
-        msg_id = self.client.shell_channel.complete(text=text, line=line,
-                                                        cursor_pos=cursor_pos)
-        
+        # Give the kernel up to 5s to respond
+        msg_id = self.client.complete(
+            code=line,
+            cursor_pos=cursor_pos,
+        )
+    
         msg = self.client.shell_channel.get_msg(timeout=self.timeout)
         if msg['parent_header']['msg_id'] == msg_id:
-            return msg["content"]["matches"]
+            content = msg['content']
+            cursor_start = content['cursor_start']
+            matches = [ line[:cursor_start] + m for m in content['matches'] ]
+            if content["cursor_end"] < cursor_pos:
+                extra = line[content["cursor_end"]: cursor_pos]
+                matches = [m + extra for m in matches]
+            matches = [ unicode_to_str(m) for m in matches ]
+            return matches
         return []
     
     def rlcomplete(self, text, state):
